clear all;

k=2; % Number of clusters

%% --- Read the small image, extract the color information of every pixel
A = double(imread('William-small.png')); 
n = size(A,1)*size(A,2);                       % number of pixels
R = A(:,:,1);                                  % red information 
G = A(:,:,2);                                  % green information 
B = A(:,:,3);                                  % blue information 
data = [R(:) G(:) B(:)];

%% --- Initialize the centers of clusters by randomly choose from the photo
C = cell(k,1);

X1 = round(size(A,1)*rand(k,1));
X2 = round(size(A,2)*rand(k,1));

for i=1:k
    temp = A(X1(i),X2(i),:,:);
    C{i} = temp(:);
end
Centre = [C{:}]';                               % initial clusters

% delete unnecessary variables
clear R G B;
clear C;
clear i temp X1 X2;

%% --- Set stop conditions
SSE_2 = 1e10;
SSE_1 = 1e9;

counter = 0;
max_iter = 200;

%% --- Compute the centers by iteration
while SSE_1 < SSE_2 && counter < max_iter
    disp(['number of iteration: ', num2str(counter)]);
    
    % find the closest center for every pixel
    P = zeros(n,1);
    temp = zeros(k,size(A,3));
    N = zeros(k,1);
    s = zeros(k,1);
    for j=1:n
        v = (Centre - repmat(data(j,:),k,1))';
        v = sum(v.^2);
        [~,ind] = min(v);
        P(j) = ind;
    end
    
    % compute the new center
    for j=1:n
        temp(P(j),:) = temp(P(j),:) + data(j,:);
        N(P(j)) = N(P(j)) + 1;
    end
    
    Centre = temp./(repmat(N,1,size(A,3)));
    
    for j=1:n
        s(P(j)) = s(P(j)) + norm(Centre(P(j),:)-data(j,:))^2;
    end
    
    SSE_2 = SSE_1;
    SSE_1 = sum(s);
    
    counter = counter + 1;
end

Centre = round(Centre);                             % final center

%% --- Import the photo by using the centers above
large_image = double(imread('William-big.png')); 
r1 = size(large_image,1);
r2 = size(large_image,2);

for i=1:r1
    for j=1:r2
        % find the closest center for every pixel in the William-big image.
        r = large_image(i,j,1); 
        g = large_image(i,j,2); 
        b = large_image(i,j,3);
        diff = repmat([r g b],k,1) - Centre;
        [~,ind] = min(sum(diff.^2,2));              % record the index of center
        large_image(i,j,:) = Centre(ind,:);         % replace the color information by the computed center
    end
end

figure;
imshow(uint8(round(large_image)));






